@@ -1,6 +1,6 @@
 from ..api import APIWrapper
 from .errors import ProxmoxException
-from typing import Dict, Any
+from typing import Dict, Any, List, Tuple
 from random import choice
 
 
@@ -59,36 +59,61 @@ class ProxmoxNodeDict:
         self._get_nodes()
         return self._nodes.items()
 
-    def choose_at_random(self, online_only: bool = True) -> ProxmoxNode:
+    def choose_at_random(self, online_only: bool = True,  nodes: List[ProxmoxNode] = None) -> ProxmoxNode:
         """
         Choose random node from list of availible nodes
         :param online_only: Only choose between nodes that are currently online (optional, default=True)
+        :param nodes: Only choose between a given list of nodes (optional)
         :return: ProxmoxNode object
         """
-        valid_choices = [node for node in self.values() if node.online or not online_only]
+        if nodes is None:
+            nodes = self.values()
+
+        valid_choices = [node for node in nodes if node.online or not online_only]
         if not valid_choices:
             raise ProxmoxException(f"No {'online ' if online_only else ''}nodes found")
         return choice(valid_choices)
 
-    def choose_by_most_free_ram(self, absolute: bool = True, online_only: bool = True):
+    @staticmethod
+    def get_memory_info(nodes: List[ProxmoxNode]) -> List[Tuple[ProxmoxNode, float, float]]:
+        """
+        Get memory info for a specific list of nodes
+        :param nodes: list of ProxmoxNode objects
+        :return: A list of tuples (ProxmoxNode, [free memory (float)], [fraction of free memory (float)])
+        """
+        result = []
+
+        for node in nodes:
+            memory_info = node.get_status_report()["memory"]
+            rating_abs = float(memory_info["free"])
+            rating = rating_abs / float(memory_info["total"])
+            result.append((node, rating_abs, rating))
+
+        return result
+
+    def choose_by_most_free_ram(self, absolute: bool = True, online_only: bool = True, nodes: List[ProxmoxNode] = None) -> ProxmoxNode:
         """
         Choose from list of availible nodes with most free RAM
         :param absolute: Whether to rate free RAM in bytes or % (optional, default=True)
         :param online_only: Only choose between nodes that are currently online (optional, default=True)
+        :param nodes: Only choose between a given list of nodes (optional)
         :return: ProxmoxNode object
         """
-        valid_choices = [node for node in self.values() if node.online or not online_only]
+        if nodes is None:
+            nodes = self.values()
+
+        valid_choices = [node for node in nodes if node.online or not online_only]
         if not valid_choices:
             raise ProxmoxException(f"No {'online ' if online_only else ''}nodes found")
-        best_rating = 0.0
-        best_node = valid_choices[0]
-        for node in valid_choices[1:]:
-            memory_info = node.get_status_report()["memory"]
-            rating = float(memory_info["free"])
-            if not absolute:
-                rating /= float(memory_info["total"])
-            if rating > best_rating:
-                best_node = node
+
+        memory_info = self.get_memory_info(valid_choices)
+
+        rating_index = 1 if absolute else 2
+
+        nodes_sorted = sorted(memory_info, key=lambda result: result[rating_index], reverse=True)
+        best_node_info = nodes_sorted[0]
+        best_node = best_node_info[0]
+
         return best_node
 
     def __len__(self):
